# 
# copy-from-archive-cache.sh - try to get an archive cached rvsn
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the University
# of California
# 
# See the file "COPYING" for further information about
# the copyright and warranty status of this work.
# 

set -e 

command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "try to copy a revision from an archive\'s cache\\n"
                printf "usage: copy-from-archive-cache [options] revision\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "Look for a cached copy of REVISION in an archive.  If one is found,\\n"
                printf "copy it to the current directory and exit with status 0.  Otherwise,\\n"
                printf "exit with status 1.\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

while test $# -ne 0 ; do

  case "$1" in 

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "copy-from-archive-cache: debugging output enabled\\n"
                    set -x
                    debug_opt=--debug
                    ;;

    --)                        shift
                        break
                        ;;

    -*)                        printf "copy-from-archive-cache: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;

  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
  printf "usage: copy-from-archive-cache [options] revision\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

revision="$1"
shift

################################################################
# Sanity Check and Process Defaults
# 

arx valid-package-name -l -e copy-from-archive-cache -- "$revision"

category=`arx parse-package-name -b $revision`
branch=`arx parse-package-name $revision`
vsn=`arx parse-package-name -v $revision`
lvl=`arx parse-package-name -l $revision`

################################################################
# Greetings
# 
# This is an internal command.
# 

if test $verbosity -ge $report ; then
  ARCH__OUTLINE_DEPTH="$ARCH__OUTLINE_DEPTH*"
  arx heading "copy-from-archive-cache\\n"
  printf "arguments: %s\\n" "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose ; then
    arx heading --sub "copy-from-archive-cache start time: %s\\n" "`date`"
    arx heading --sub "seeking revision: %s\\n" "$branch--$vsn--$lvl"
    arx heading --sub "from archive: %s\\n" "$WITHARCHIVE"
    arx heading --sub "to store in: %s\\n" "`pwd`"
  fi
fi

################################################################
# Look for Cached Copies
# 

wftp-home
wftp-cd "$category/$branch/$branch--$vsn/$lvl"

if ! wftp-get $branch--$vsn--$lvl.tar.gz > ,,$branch--$vsn--$lvl.tar.gz 2> /dev/null ; then

  rm -f ,,$branch--$vsn--$lvl.tar.gz

  if test $verbosity -ge $quiet; then
    arx heading --sub "no cached copy found\\n"
    if test $verbosity -ge $verbose; then
      arx heading --sub "copy-from-archive-cache finish time: %s\\n" "`date`"
    fi
  fi
  exit 1

else

  if test $verbosity -ge $report; then
    arx heading --sub "revision found cached in archive: %s\\n" $branch--$vsn--$lvl
  fi

  here="`pwd`"
  rm -rf ,,$branch--$vsn--$lvl
  mkdir ,,$branch--$vsn--$lvl
  cd  ,,$branch--$vsn--$lvl

  if test $verbosity -ge $verbose; then
    tar -m -zvxf ../,,$branch--$vsn--$lvl.tar.gz | arx body-indent --sub
  else
    tar -m -zxf ../,,$branch--$vsn--$lvl.tar.gz
  fi

  cd $branch--$vsn--$lvl
  for f in `ls -a | grep -v -E "^\\.\\.?\$"` ; do
    mv "$f" "$here"
  done
  cd "$here"
  rm -rf ",,$branch--$vsn--$lvl"
  rm -rf ",,$branch--$vsn--$lvl.tar.gz"

fi

