# 
# versions.sh - print a list of versions in an archive branch
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# Minor Bug Fix 2003 Costa 
# 
# See the file "COPYING" for further information about
# the copyright and warranty status of this work.
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "list the versions in an archive branch\\n"
                printf "usage: versions [options] [branch]\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -r --reverse                  sort from newest to oldest\\n"
		printf " --full                        print the full names including the archive\\n"
                printf "\\n"
                printf "Print a list of versions within an archive branch.\\n"
                printf "\\n"
                printf "The list is ordinarilly sorted from oldest to newest,\\n"
                printf "but the order can be changed with -r (--reverse).\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=
reverse=
debug_opt=
full=

while test $# -ne 0 ; do

  case "$1" in 

    --debug)                shift
                            debug_opt=--debug
                        printf "\n" 1>&2
                        printf "versions: DEBUGGING ACTIVATED\n" 1>&2
                        printf "\n" 1>&2
                        set -x
                        ;;

    -r|--reverse)        shift
                            reverse=-r
                        ;;

    --full)             shift
	                full="--full"
			;;

    -*)                        printf "versions: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    "")                 shift
	                ;;

    *)                        break
                            ;;

  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -gt 1 ; then
  printf "usage: versions [options] [branch]\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

if test $# -ne 0 ; then
  branch="$1"
  shift
else
  branch=
fi

################################################################
# Sanity Check and Process Defaults
# 

if test -z "$branch" ; then
  branch="`arx tree-version .`"
fi

arx valid-package-name -t -e versions -- "$branch"
archive="`arx parse-package-name --arch \"$branch\"`"
category=`arx parse-package-name --basename "$branch"`
branch=`arx parse-package-name --package "$branch"`


################################################################
# Establish an Archive Connection
# 

if test "$WITHARCHIVE" != "$archive" ; then
  exec arx with-archive -A "$archive" arx versions $full $debug_opt $reverse "$archive/$branch"
fi

################################################################
# Print the List
# 

basere="([a-zA-Z]([a-zA-Z0-9]*(-[a-zA-Z0-9]+)?)*)"
vsnre="([0-9]+\\.[0-9]+)"

wftp-home

if ! wftp-cd "$category/$branch" ; then
  printf "\\n" 1>&2
  printf "versions: no such branch\\n" 1>&2
  printf "  archive: %s\\n" "$archive" 1>&2
  printf "  branch: %s\\n" "$branch" 1>&2
  printf "\\n" 1>&2
  exit 1
fi

versions=`wftp-ls \
| grep -E "^($basere)(--$basere)?(--$vsnre)$" \
| sed -e "s/.*--\\([0-9].*\\)/\\1.&/" \
| sort  -t. -k 1,1${reverse#-}n -k 2,2${reverse#-}n \
| sed -e "s/[^.]*.[^.]*.//"`

for version in $versions ; do
  if test -n "$full" ; then
    printf "%s/%s\\n" $archive $version
  else
    printf "%s\\n" $version
  fi
done

