# file-history.sh: report the history of all files
# 
################################################################
# Copyright (C) 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the
#                                  University of California
# 
# See the file "COPYING" for further information about
# the copyright and warranty status of this work.
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "report the history of all files\\n"
                printf "usage: file-history [options] [[archive]/version]"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -r --reverse                  reverse the listing\\n"
                printf "\\n"
                printf "Print a list of all files in a given version, along with an indication\\n"
                printf "of what patches modified, added, removed, or deleted them.\\n"
                printf "\\n"
                printf "The output format contains a section for each file:\\n"
                printf "\\n"
                printf "        TAG\\n"
                printf "                PATCH        [ADMR]        [NAME]\\n"
                printf "                PATCH        [ADMR]        [NAME]\\n"
                printf "                ...\\n"
                printf "\\n"
                printf "The PATCH field is a patch level where the file was affected\\n"
                printf "in some way.  The next field is some non-empty string of letters\\n"
                printf "from [ADMR] (added, deleted, modified, renamed).\\n"
                printf "\\n"
                printf "If the file was added or renamed, the new name is included in\\n"
                printf "the third field which is otherwise empty.\\n"
                printf "\\n"
                printf "The indented fields are sorted patch level.\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=

reverse=

while test $# -ne 0 ; do

  case "$1" in 

    -r|--reverse)        shift
                            reverse=-r
                        ;;

    -*)                 printf "file-history: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
  printf "usage: file-history [options] [archive]/version\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

if test $# -gt 0 ; then
  versionspec="$1"
  shift
fi

################################################################
# Sanity Check and Process Defaults
# 

library="`arx my-revision-library -e file-history`"

arx valid-package-name -e file-history --vsn "$versionspec"
archive="`arx parse-package-name --arch \"$versionspec\"`"
version="`arx parse-package-name --non-arch \"$versionspec\"`"

libdir="`arx library-find \"$archive/$version--base-0\"`"
cd "$libdir"
cd ..
libdir="`pwd`"


################################################################
# Do It
# 

arx library-revisions $reverse --full "$archive/$version" \
| awk -f "$ARCH_SUBCMD_ROOT/library/file-history.awk"

# tag: Tom Lord Sun Jan 20 01:11:00 2002 (library/file-history.sh)
#
