# library-find.sh: find a revision in a revision library
# 
################################################################
# Copyright (C) 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the
#                                  University of California
# 
# See the file "COPYING" for further information about
# the copyright and warranty status of this work.
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "find a revision in a revision library\\n"
                printf "usage: library-find [options] [archive/]revision\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf " -e --errname NAME             use NAME in error messages\\n"
                printf "\\n"
                printf "Find REVISION in your revision library and print its path.\\n"
                printf "If the revision is not present, print an error message (unless\\n"
                printf "\"--silent\" is specified) and exit with non-0 status.\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=

errname=library-find

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

while test $# -ne 0 ; do

  case "$1" in 

    -e|--errname)                shift
                                if test $# -eq 0 ; then
                                  printf "library-find: -e and --errname require an argument\\n" 1>&2
                                  printf "try --help\\n" 1>&2
                                  exit 1
                                fi
                                errname="$1"
                                shift
                                ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "library-find: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    -*)                 printf "library-find: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
  printf "usage: library-find [options] [archive/]revision\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

rvnspec="$1"
shift

################################################################
# Sanity Check and Process Defaults
# 

library="`arx my-revision-library -e library-find 2> /dev/null`"

archive_revision=`arx indicated-revision -e library-find "$rvnspec"`

archive="`arx parse-package-name --arch \"$archive_revision\"`"
revision="`arx parse-package-name --non-arch \"$archive_revision\"`"
category="`arx parse-package-name --basename \"$revision\"`"
branch="`arx parse-package-name \"$revision\"`"
version="`arx parse-package-name --package-version \"$revision\"`"

################################################################
# Do It
# 

if test -d "$library/$archive/$category/$branch/$version/$revision" ; then
  if test $verbosity -ge $default_output ; then
    printf "%s\\n"  "$library/$archive/$category/$branch/$version/$revision"
  fi
  exit 0
else
  if test $verbosity -ge $quiet ; then
    printf "\\n" 1>&2
    printf "%s: revision not in library\\n" "$errname" 1>&2
    printf "  archive: %s\\n" "$archive" 1>&2
    printf "  revision: %s\\n" "$revision" 1>&2
    printf "\\n" 1>&2
  fi
  exit 1
fi  


# tag: Tom Lord Sun Jan 20 00:09:29 2002 (library/library-find.sh)
#
